import expect from 'expect'
import approvalRoute from '../../src/reducers'

describe('approvalRoute reducer', () => {
	it('should handle initial state', () => {
	  expect(
	    approvalRoute(undefined, {})
	  ).toEqual([])
	})
	
	it('should handle ADD_ROUTE_ITEM', () => {
		expect(
			approvalRoute([], {
				type: 'ADD_ROUTE_ITEM',
				id: 0,
				itemType: 'approver',
				data: { employeeName: 'Kirkorov' }
			})
		).toEqual([
			{
				id: 0,
				itemType: 'approver', 
				data: { employeeName: 'Kirkorov' } 
			}
		], "добавление в пустой маршрут")

		expect(
			approvalRoute([
				{
					id: 0,
					itemType: 'approver', 
					data: { employeeName: 'Kirkorov' } 
				},
				{
					id: 3,
					itemType: 'stage', 
					data: {prevItemID: 0}
				},
				{
					id: 1,
					itemType: 'approver', 
					data: { employeeName: 'Pugacheva' } 
				}
			], {
				type: 'ADD_ROUTE_ITEM',
				id: 4,
				itemType: 'approver',
				data: { employeeName: 'Galkin' }
			})
		).toEqual([
			{
				id: 0,
				itemType: 'approver', 
				data: { employeeName: 'Kirkorov' } 
			},
			{
				id: 3,
				itemType: 'stage', 
				data: {prevItemID: 0}
			},
			{
				id: 1,
				itemType: 'approver', 
				data: { employeeName: 'Pugacheva' } 
			},
			{
				id: 4,
				itemType: 'approver', 
				data: { employeeName: 'Galkin' } 
			}
		], "добавление в заполненный маршрут")	

		expect(
			approvalRoute([
				{
					id: 0,
					itemType: 'approver', 
					data: { employeeName: 'Kirkorov' } 
				},
				{
					id: 1,
					itemType: 'approver', 
					data: { employeeName: 'Pugacheva' } 
				},
				{
					id: 5,
					itemType: 'stage', 
					data: {prevItemID: 1}
				},
				{
					id: 4,
					itemType: 'approver', 
					data: { employeeName: 'Galkin' } 
				}
			], {
				type: 'ADD_ROUTE_ITEM',
				id: 3,
				itemType: 'stage',
				data: {prevItemID: 0}
			})
		).toEqual([
			{
				id: 0,
				itemType: 'approver', 
				data: { employeeName: 'Kirkorov' } 
			},
			{
				id: 3,
				itemType: 'stage', 
				data: {prevItemID: 0}
			},
			{
				id: 1,
				itemType: 'approver', 
				data: { employeeName: 'Pugacheva' } 
			},
			{
				id: 5,
				itemType: 'stage', 
				data: {prevItemID: 1}
			},
			{
				id: 4,
				itemType: 'approver', 
				data: { employeeName: 'Galkin' } 
			}
		], "разбивание на этапы")
	})

	it('should handle REMOVE_ROUTE_ITEM', () => {
		expect(
			approvalRoute([ 
				{
					id: 0,
					itemType: 'approver', 
				}
			], 
			{
				type: 'REMOVE_ROUTE_ITEM',
				id: 0
			})
		).toEqual([], 'удаление последнего сотрудника')

		expect(
			approvalRoute([
				{
					id: 0,
					itemType: 'approver', 
				},
				{
					id: 1,
					itemType: 'approver', 
				},
				{
					id: 4,
					itemType: 'approver', 
				}
			], {
				type: 'REMOVE_ROUTE_ITEM',
				id: 1
			})
		).toEqual([
			{
				id: 0,
				itemType: 'approver', 
			},
			{
				id: 4,
				itemType: 'approver', 
			}
		], 'удаление одного из сотрудников')

		expect(
			approvalRoute([
				{
					id: 0,
					itemType: 'approver', 
				},
				{
					id: 3,
					itemType: 'stage', 
				},
				{
					id: 1,
					itemType: 'approver', 
				},
				{
					id: 4,
					itemType: 'approver', 
				}
			], {
				type: 'REMOVE_ROUTE_ITEM',
				id: 0
			})
		).toEqual([
			{
				id: 1,
				itemType: 'approver', 
			},
			{
				id: 4,
				itemType: 'approver', 
			}
		], 'удаление единственного сотрудника из первого этапа')

		expect(
			approvalRoute([
				{
					id: 0,
					itemType: 'approver', 
				},
				{
					id: 1,
					itemType: 'approver', 
				},
				{
					id: 3,
					itemType: 'stage', 
				},
				{
					id: 4,
					itemType: 'approver', 
				}
			], {
				type: 'REMOVE_ROUTE_ITEM',
				id: 4
			})
		).toEqual([
			{
				id: 0,
				itemType: 'approver', 
			},
			{
				id: 1,
				itemType: 'approver', 
			}
		], 'удаление единственного сотрудника из последнего этапа')

		expect(
			approvalRoute([
				{
					id: 0,
					itemType: 'approver', 
				},
				{
					id: 1,
					itemType: 'approver', 
				},
				{
					id: 3,
					itemType: 'stage', 
				},
				{
					id: 4,
					itemType: 'approver', 
				},
				{
					id: 5,
					itemType: 'stage', 
				},
				{
					id: 6,
					itemType: 'approver', 
				},
				{
					id: 7,
					itemType: 'approver', 
				}
			], {
				type: 'REMOVE_ROUTE_ITEM',
				id: 4
			})
		).toEqual([
			{
				id: 0,
				itemType: 'approver', 
			},
			{
				id: 1,
				itemType: 'approver', 
			},
			{
				id: 3,
				itemType: 'stage', 
			},
			{
				id: 6,
				itemType: 'approver', 
			},
			{
				id: 7,
				itemType: 'approver', 
			}
		], 'удаление единственного сотрудника из промежуточного этапа')
	})

	// it('should handle MOVE_APPROVER', () => {
	// 	expect(
	// 		approvalRoute([
	// 			[
	// 				{
	// 					approver: '1'
	// 				},
	// 				{
	// 					approver: '2'
	// 				}
	// 			]
	// 		], {
	// 			type: 'MOVE_APPROVER',
	// 			stage: 0,
	// 			index: 1,
	// 			to: 'down'
	// 		})
	// 	).toEqual([
	// 		[
	// 			{
	// 				approver: '1'
	// 			}
	// 		],
	// 		[
	// 			{
	// 				approver: '2'
	// 			}
	// 		]
	// 	], 'CREATE STAGE')

	// 	expect(
	// 		approvalRoute([
	// 			[
	// 				{
	// 					approver: '1'
	// 				}
	// 			],
	// 			[
	// 				{
	// 					approver: '2'
	// 				}
	// 			]
	// 		], {
	// 			type: 'MOVE_APPROVER',
	// 			stage: 1,
	// 			index: 0,
	// 			to: 'down'
	// 		})
	// 	).toEqual([
	// 		[
	// 			{
	// 				approver: '1'
	// 			}
	// 		],
	// 		[
	// 			{
	// 				approver: '2'
	// 			}
	// 		]
	// 	], 'SAVE STATE')

	// 	expect(
	// 		approvalRoute([
	// 			[
	// 				{
	// 					approver: '1'
	// 				},
	// 				{
	// 					approver: '2'
	// 				},
	// 				{
	// 					approver: '3'
	// 				}
	// 			],
	// 			[
	// 				{
	// 					approver: '4'
	// 				}
	// 			]
	// 		], {
	// 			type: 'MOVE_APPROVER',
	// 			stage: 0,
	// 			index: 2,
	// 			to: 'down'
	// 		})
	// 	).toEqual([
	// 		[
	// 			{
	// 				approver: '1'
	// 			},
	// 			{
	// 				approver: '2'
	// 			}
	// 		],
	// 		[
	// 			{
	// 				approver: '3'
	// 			},
	// 			{
	// 				approver: '4'
	// 			}
	// 		]
	// 	])

	// 	expect(
	// 		approvalRoute([
	// 			[
	// 				{
	// 					approver: '1'
	// 				},
	// 				{
	// 					approver: '2'
	// 				}
	// 			],
	// 			[
	// 				{
	// 					approver: '3'
	// 				},
	// 				{
	// 					approver: '4'
	// 				}
	// 			]
	// 		], {
	// 			type: 'MOVE_APPROVER',
	// 			stage: 0,
	// 			index: 1,
	// 			to: 'down'
	// 		})
	// 	).toEqual([
	// 		[
	// 			{
	// 				approver: '1'
	// 			}
	// 		],
	// 		[
	// 			{
	// 				approver: '2'
	// 			},
	// 			{
	// 				approver: '3'
	// 			},
	// 			{
	// 				approver: '4'
	// 			}
	// 		]
	// 	])

	// 	expect(
	// 		approvalRoute([
	// 			[
	// 				{
	// 					approver: '1'
	// 				}
	// 			],
	// 			[
	// 				{
	// 					approver: '2'
	// 				},
	// 				{
	// 					approver: '3'
	// 				},
	// 				{
	// 					approver: '4'
	// 				}
	// 			]
	// 		], {
	// 			type: 'MOVE_APPROVER',
	// 			stage: 0,
	// 			index: 0,
	// 			to: 'down'
	// 		})
	// 	).toEqual([
	// 		[
	// 			{
	// 				approver: '1'
	// 			},
	// 			{
	// 				approver: '2'
	// 			},
	// 			{
	// 				approver: '3'
	// 			},
	// 			{
	// 				approver: '4'
	// 			}
	// 		]
	// 	])
	// })

	// it('should handle MOVE_UP_APPROVER', () => {
	// 	expect(
	// 		approvalRoute([
	// 			[
	// 				{
	// 					approver: '1'
	// 				}
	// 			],
	// 			[
	// 				{
	// 					approver: '2'
	// 				}
	// 			]
	// 		], {
	// 			type: 'MOVE_APPROVER',
	// 			stage: 0,
	// 			index: 0,
	// 			to: 'up'
	// 		})
	// 	).toEqual([
	// 		[
	// 			{
	// 				approver: '1'
	// 			}
	// 		],
	// 		[
	// 			{
	// 				approver: '2'
	// 			}
	// 		]
	// 	])

	// 	expect(
	// 		approvalRoute([
	// 			[
	// 				{
	// 					approver: '1'
	// 				},
	// 				{
	// 					approver: '2'
	// 				},
	// 				{
	// 					approver: '3'
	// 				}
	// 			]
	// 		], {
	// 			type: 'MOVE_APPROVER',
	// 			stage: 0,
	// 			index: 0,
	// 			to: 'up'
	// 		})
	// 	).toEqual([
	// 		[
	// 			{
	// 				approver: '1'
	// 			}
	// 		],
	// 		[
	// 			{
	// 				approver: '2'
	// 			},
	// 			{
	// 				approver: '3'
	// 			}
	// 		]
	// 	])

	// 	expect(
	// 		approvalRoute([
	// 			[
	// 				{
	// 					approver: '1'
	// 				}
	// 			],
	// 			[
	// 				{
	// 					approver: '2'
	// 				},
	// 				{
	// 					approver: '3'
	// 				},
	// 				{
	// 					approver: '4'
	// 				}
	// 			]
	// 		], {
	// 			type: 'MOVE_APPROVER',
	// 			stage: 1,
	// 			index: 0,
	// 			to: 'up'
	// 		})
	// 	).toEqual([
	// 		[
	// 			{
	// 				approver: '1'
	// 			},
	// 			{
	// 				approver: '2'
	// 			}
	// 		],
	// 		[
	// 			{
	// 				approver: '3'
	// 			},
	// 			{
	// 				approver: '4'
	// 			}
	// 		]
	// 	])

	// 	expect(
	// 		approvalRoute([
	// 			[
	// 				{
	// 					approver: '1'
	// 				},
	// 				{
	// 					approver: '2'
	// 				},
	// 				{
	// 					approver: '3'
	// 				}
	// 			],
	// 			[
	// 				{
	// 					approver: '4'
	// 				}
	// 			]
	// 		], {
	// 			type: 'MOVE_APPROVER',
	// 			stage: 0,
	// 			index: 2,
	// 			to: 'up'
	// 		})
	// 	).toEqual([
	// 		[
	// 			{
	// 				approver: '3'
	// 			}
	// 		],
	// 		[
	// 			{
	// 				approver: '1'
	// 			},
	// 			{
	// 				approver: '2'
	// 			},
	// 		],
	// 		[
	// 			{
	// 				approver: '4'
	// 			}
	// 		]
	// 	])
	// })

	// it('should handle ADD_TO', () => {
	// 	expect(
	// 		approvalRoute([
	// 			[
	// 				{approver: '1'}, {approver: '2'}, {approver: '3'}
	// 			],
	// 			[
	// 				{approver: '4'}
	// 			]
	// 		], {
	// 			type: 'ADD_TO',
	// 			stage: 1,
	// 			index: 0,
	// 			approver: '5'
	// 		})
	// 	).toEqual([
	// 		[
	// 			{approver: '1'}, {approver: '2'}, {approver: '3'}
	// 		],
	// 		[
	// 			{approver: '5'}, {approver: '4'}
	// 		]
	// 	])
	// })

	it('should handle MOVE_DOWN_STAGE', () => {
		expect(
			approvalRoute([
				{
					id: 0,
					itemType: 'approver', 
				},
				{
					id: 3,
					itemType: 'stage', 
				},
				{
					id: 1,
					itemType: 'approver', 
				},
				{
					id: 4,
					itemType: 'approver', 
				}
			], {
				type: 'MOVE_DOWN_STAGE',
				id: 3
			})
		).toEqual([
			{
				id: 1,
				itemType: 'approver', 
			},
			{
				id: 4,
				itemType: 'approver', 
			},
			{
				id: 3,
				itemType: 'stage', 
			},
			{
				id: 0,
				itemType: 'approver', 
			}
		], 'Список с одним разделителем')

		//Список с несколькими разделителями, меняются два последних
		expect(
			approvalRoute([
				{
					id: 0,
					itemType: 'approver', 
				},
				{
					id: 1,
					itemType: 'approver', 
				},
				{
					id: 3,
					itemType: 'stage', 
				},
				{
					id: 4,
					itemType: 'approver', 
				},
				{
					id: 5,
					itemType: 'stage', 
				},
				{
					id: 6,
					itemType: 'approver', 
				},
				{
					id: 7,
					itemType: 'approver', 
				}
			], {
				type: 'MOVE_DOWN_STAGE',
				id: 5
			})
		).toEqual([
			{
				id: 0,
				itemType: 'approver', 
			},
			{
				id: 1,
				itemType: 'approver', 
			},
			{
				id: 3,
				itemType: 'stage', 
			},
			{
				id: 6,
				itemType: 'approver', 
			},
			{
				id: 7,
				itemType: 'approver', 
			},
			{
				id: 5,
				itemType: 'stage', 
			},
			{
				id: 4,
				itemType: 'approver', 
			}
		], 'Список с несколькими разделителями, меняются два последних')

		//Список с несколькими разделителями, меняются два первых
		expect(
			approvalRoute([
				{
					id: 0,
					itemType: 'approver', 
				},
				{
					id: 1,
					itemType: 'approver', 
				},
				{
					id: 3,
					itemType: 'stage', 
				},
				{
					id: 4,
					itemType: 'approver', 
				},
				{
					id: 5,
					itemType: 'stage', 
				},
				{
					id: 6,
					itemType: 'approver', 
				},
				{
					id: 7,
					itemType: 'approver', 
				}
			], {
				type: 'MOVE_DOWN_STAGE',
				id: 3
			})
		).toEqual([
			{
				id: 4,
				itemType: 'approver', 
			},
			{
				id: 3,
				itemType: 'stage', 
			},
			{
				id: 0,
				itemType: 'approver', 
			},
			{
				id: 1,
				itemType: 'approver', 
			},
			{
				id: 5,
				itemType: 'stage', 
			},
			{
				id: 6,
				itemType: 'approver', 
			},
			{
				id: 7,
				itemType: 'approver', 
			}
		], 'Список с несколькими разделителями, меняются два первых')

		// Список с несколькими разделителями, меняются средние
		expect(
			approvalRoute([
				{
					id: 4,
					itemType: 'approver', 
				},
				{
					id: 3,
					itemType: 'stage', 
				},
				{
					id: 0,
					itemType: 'approver', 
				},
				{
					id: 8,
					itemType: 'stage', 
				},
				{
					id: 1,
					itemType: 'approver', 
				},
				{
					id: 5,
					itemType: 'stage', 
				},
				{
					id: 6,
					itemType: 'approver', 
				},
				{
					id: 7,
					itemType: 'approver', 
				}
			], {
				type: 'MOVE_DOWN_STAGE',
				id: 8
			})
		).toEqual([
			{
				id: 4,
				itemType: 'approver', 
			},
			{
				id: 3,
				itemType: 'stage', 
			},
			{
				id: 1,
				itemType: 'approver', 
			},
			{
				id: 8,
				itemType: 'stage', 
			},
			{
				id: 0,
				itemType: 'approver', 
			},
			{
				id: 5,
				itemType: 'stage', 
			},
			{
				id: 6,
				itemType: 'approver', 
			},
			{
				id: 7,
				itemType: 'approver', 
			}
		], 'Список с несколькими разделителями, меняются средние')
	})

	it('should handle UNION_GROUP (~REMOVE STAGE)', () => {
		expect(
			approvalRoute([
				{
					id: 0,
					itemType: 'approver', 
					data: { employeeName: 'Kirkorov' } 
				},
				{
					id: 3,
					itemType: 'stage', 
					data: {prevItemID: 0}
				},
				{
					id: 1,
					itemType: 'approver', 
					data: { employeeName: 'Pugacheva' } 
				}
			], {
				type: 'REMOVE_ROUTE_ITEM',
				id: 3
			})
		).toEqual([
			{
				id: 0,
				itemType: 'approver', 
				data: { employeeName: 'Kirkorov' } 
			},
			{
				id: 1,
				itemType: 'approver', 
				data: { employeeName: 'Pugacheva' } 
			}
		])
	})

})